package meerkat.bulletinboard.httpserver;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import meerkat.bulletinboard.BulletinBoardServer;
import meerkat.bulletinboard.sqlserver.SQLiteBulletinBoardServer;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.BulletinBoardAPI.*;

public class BulletinBoardHttpServer extends HttpServlet {

	public final static File DEFAULT_MEERKAT_DB = new File("local-instances/meerkat.db");

	/**
	 * Auto-generated UID. 
	 */
	private static final long serialVersionUID = -1263665607729456165L;
	
	BulletinBoardServer bbs;
	
	@Override
	public void init(ServletConfig config) throws ServletException {
		//TODO: Make this generic
		bbs = new SQLiteBulletinBoardServer();
		
		try {
			bbs.init(DEFAULT_MEERKAT_DB);
		} catch (CommunicationException e) {
			// TODO Log error
			throw new ServletException("Servlet failed to initialize: " + e.getMessage());
		}
	}
	
	/**
	 * This procedure handles (POST) requests to post messages to the Bulletin Board.
	 */
	@Override
	protected void doPost( HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		
		BulletinBoardMessage message;
		
		try{
			message = BulletinBoardMessage.newBuilder()
				.mergeFrom(request.getInputStream())
				.build();
		} catch(Exception e){
			//TODO: Log invalid request
			return;
		}
		
		try {
			bbs.postMessage(message);
		} catch (CommunicationException e) {
			// TODO Log DB communication error
		}
		
	}
	
	/**
	 * This procedure handles (GET) requests which request data from the Bulletin Board.
	 */
	@Override
	protected void doGet( HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		
		BulletinBoardMessage message;
		
		try{
			message = BulletinBoardMessage.newBuilder()
				.mergeFrom(request.getInputStream())
				.build();
		} catch(Exception e){
			//TODO: Log invalid request
			return;
		}
		
		try {
			bbs.postMessage(message);
		} catch (CommunicationException e) {
			// TODO Log DB communication error
		}
		
	}
	
	@Override
	public void destroy() {
		
		try {
			bbs.close();
		} catch (CommunicationException e) {
			// TODO Log DB communication error
		}
		
	}
	
}
