package meerkat.bulletinboard.webapp;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.ws.rs.Consumes;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.MediaType;

import meerkat.bulletinboard.BulletinBoardServer;
import meerkat.bulletinboard.sqlserver.SQLiteBulletinBoardServer;
import meerkat.comm.CommunicationException;
import meerkat.protobuf.BulletinBoardAPI.BoolMsg;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessageList;
import meerkat.protobuf.BulletinBoardAPI.MessageFilterList;
import meerkat.rest.Constants;

import java.io.File;

@Path("/sqlserver")
public class BulletinBoardWebApp implements BulletinBoardServer {
	
	BulletinBoardServer bulletinBoard;
	
	@PostConstruct
	@Override
	public void init(File meerkatDB) throws CommunicationException {
		bulletinBoard = new SQLiteBulletinBoardServer();
		bulletinBoard.init(meerkatDB);
	}
	
	@Path("postmessage")
	@POST
	@Consumes(Constants.MEDIATYPE_PROTOBUF)
	@Produces(Constants.MEDIATYPE_PROTOBUF)
	@Override
	public BoolMsg postMessage(BulletinBoardMessage msg) throws CommunicationException {
		return bulletinBoard.postMessage(msg);
	}
	
	@Path("readmessages")
	@POST
	@Consumes(Constants.MEDIATYPE_PROTOBUF)
	@Produces(Constants.MEDIATYPE_PROTOBUF)
	@Override
	public BulletinBoardMessageList readMessages(MessageFilterList filterList) throws CommunicationException {
		return bulletinBoard.readMessages(filterList);
	}

	@Override
	@PreDestroy
	public void close() throws CommunicationException {
		bulletinBoard.close();
	}
	
	@GET
    @Produces(MediaType.TEXT_PLAIN)
	public String test() {
        return "This BulletinBoard is up and running!\n Please consult the API documents to perform queries.";
	}
	
}
