package meerkat.bulletinboard;


import com.google.protobuf.ByteString;
import com.google.protobuf.TextFormat;

import meerkat.protobuf.Crypto.*;
import meerkat.protobuf.BulletinBoardAPI.*;
import meerkat.rest.Constants;
import meerkat.rest.ProtobufMessageBodyReader;
import meerkat.rest.ProtobufMessageBodyWriter;

import org.junit.Before;
import org.junit.Test;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;

public class SQLiteServerIntegrationTest {

    private static String PROP_GETTY_URL = "gretty.httpBaseURI";
    private static String DEFAULT_BASE_URL = "localhost:8081";
    private static String BASE_URL =  System.getProperty(PROP_GETTY_URL, DEFAULT_BASE_URL);
    private static String SQL_SERVER_POST = "sqlserver/postmessage";
    private static String SQL_SERVER_GET = "sqlserver/readmessages";

    Client client;
//    Connection connection;

    @Before
    public void setup() throws Exception {
        client = ClientBuilder.newClient();
        client.register(ProtobufMessageBodyReader.class);
        client.register(ProtobufMessageBodyWriter.class);

    }

    @Test
    public void testPost() throws Exception {
        byte[] b1 = {(byte) 1, (byte) 2, (byte) 3, (byte) 4};
        byte[] b2 = {(byte) 11, (byte) 12, (byte) 13, (byte) 14};
        byte[] b3 = {(byte) 21, (byte) 22, (byte) 23, (byte) 24};
        byte[] b4 = {(byte) 4, (byte) 5, (byte) 100, (byte) -50, (byte) 0};

        WebTarget webTarget;
        Response response;
        BoolMsg bool;
        
        BulletinBoardMessage msg;
        
        MessageFilterList filterList; 
        BulletinBoardMessageList msgList;
        
//        try{
//			connection = DriverManager.getConnection("jdbc:sqlite:d:/arbel/projects/meerkat-java/bulletin-board-server/local-instances/meerkat.db");
//		} catch (SQLException e) {
//			System.err.println(e.getMessage());
//			assert false;
//		}
        
        // Test writing mechanism
        
        System.err.println("******** Testing: " + SQL_SERVER_POST);
        webTarget = client.target(BASE_URL).path(SQL_SERVER_POST);
        
        msg = BulletinBoardMessage.newBuilder()
                .setMsg(UnsignedBulletinBoardMessage.newBuilder()
                        .addTag("Signature")
                        .addTag("Trustee")
                        .setData(ByteString.copyFrom(b1))
                        .build())
                .addSig(Signature.newBuilder()
                        .setType(SignatureType.DSA)
                        .setData(ByteString.copyFrom(b2))
                        .setSignerId(ByteString.copyFrom(b3))
                        .build())
                .addSig(Signature.newBuilder()
                        .setType(SignatureType.ECDSA)
                        .setData(ByteString.copyFrom(b3))
                        .setSignerId(ByteString.copyFrom(b2))
                        .build())
                .build();

        response = webTarget.request(Constants.MEDIATYPE_PROTOBUF).post(Entity.entity(msg, Constants.MEDIATYPE_PROTOBUF));
        System.err.println(response);
        bool = response.readEntity(BoolMsg.class);
    	assert bool.getValue();
        
    	msg = BulletinBoardMessage.newBuilder()
                .setMsg(UnsignedBulletinBoardMessage.newBuilder()
                        .addTag("Vote")
                        .addTag("Trustee")
                        .setData(ByteString.copyFrom(b4))
                        .build())
                .addSig(Signature.newBuilder()
                        .setType(SignatureType.ECDSA)
                        .setData(ByteString.copyFrom(b4))
                        .setSignerId(ByteString.copyFrom(b2))
                        .build())
                .build();
    	
    	response = webTarget.request(Constants.MEDIATYPE_PROTOBUF).post(Entity.entity(msg, Constants.MEDIATYPE_PROTOBUF));
        System.err.println(response);
        bool = response.readEntity(BoolMsg.class);
    	assert bool.getValue();
    	
    	// Test reading mechanism
    	
    	System.err.println("******** Testing: " + SQL_SERVER_GET);
    	webTarget = client.target(BASE_URL).path(SQL_SERVER_GET);
    	
    	filterList = MessageFilterList.newBuilder()
    					.addFilter(
    							MessageFilter.newBuilder()
    							.setType(FilterType.TAG)
    							.setTag("Vote")
    							.build()
    							)
    					.build();
    	
//    	String sql = "SELECT MsgTable.EntryNum, MsgTable.Msg FROM MsgTable INNER JOIN SignatureTable ON SignatureTable.EntryNum = MsgTable.EntryNum WHERE SignatureTable.SignerId = ?";
//    	PreparedStatement pstmt = connection.prepareStatement(sql);
//    	int i=1;
//    	for (MessageFilter filter : filterList.getFilterList()){
//	    	
//	    	switch (filter.getType().getNumber()){
//	    		
//				case FilterType.EXACT_ENTRY_VALUE: // Go through.
//				case FilterType.MAX_ENTRY_VALUE:
//					pstmt.setLong(i, filter.getEntry());
//					i++;
//					break;
//				
//				case FilterType.MSG_ID_VALUE: // Go through.
//				case FilterType.SIGNER_ID_VALUE:
//					pstmt.setBytes(i, filter.getId().toByteArray());
//					i++;
//					break;
//					
//				case FilterType.TAG_VALUE:
//					pstmt.setString(i, filter.getTag());
//					break;
//				
//				// The max-messages condition is applied as a suffix. Therefore, it is treated differently.
//				case FilterType.MAX_MESSAGES_VALUE:
//					pstmt.setLong(filterList.getFilterList().size(), filter.getMaxMessages());
//					break;
//				
//	    	}
//	    }
//    	ResultSet rs = pstmt.executeQuery();
//    	
//    	i = 0;
//    	while (rs.next()){
//    		i++;
//    		assert rs.getBytes(2)
//    	}
//    	System.err.println("Local DB size = " + i);
//    	pstmt.close();
    	
    	response = webTarget.request(Constants.MEDIATYPE_PROTOBUF).post(Entity.entity(filterList, Constants.MEDIATYPE_PROTOBUF));
    	System.err.println(response);
    	msgList = response.readEntity(BulletinBoardMessageList.class);
    	System.err.println("List size: " + msgList.getMessageCount());
    	System.err.println("This is the list:");
    	System.err.println(TextFormat.printToString(msgList));
    	assert msgList.getMessageCount() == 1;
    }

}
