package meerkat.bulletinboard;

import meerkat.comm.CommunicationException;
import meerkat.protobuf.BulletinBoardAPI.*;

import java.io.File;

/**
 * Created by Arbel on 07/11/15.
 * 
 * This interface refers to a single instance of a Bulletin Board.
 * An implementation of this interface may use any DB and be hosted on any machine. 
 */

public interface BulletinBoardServer{
	
	/**
	 * This method initializes the server by reading the signature data and storing it.
	 * It also establishes the connection to the DB.
	 * @throws CommunicationException on DB connection error.
	 */
	public void init(File meerkatDB) throws CommunicationException;
	
	/**
     * Post a message to bulletin board.
     * @param msg is the actual (signed) message
	 * @return TRUE if the message has been authenticated and FALSE otherwise (in ProtoBuf form).
	 * @throws CommunicationException on DB connection error.
     */
	 public BoolMsg postMessage(BulletinBoardMessage msg) throws CommunicationException;
	 
	 /**
	  * Read all messages posted matching the given filter.
	  * @param filter return only messages that match the filter (empty list means no filtering).
	  * @return
	  */
	 BulletinBoardMessageList readMessages(MessageFilterList filterList) throws CommunicationException;
	 
	 /**
	 * This method closes the connection to the DB.
	 * @throws CommunicationException on DB connection error.
	 */
	 public void close() throws CommunicationException;
}
