package meerkat.crypto.concrete;

import org.bouncycastle.jce.provider.BouncyCastleProvider;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.security.Provider;
import java.security.Security;

/**
 * A class that performs required crypto setup
 */
public class GlobalCryptoSetup {
    final static Logger logger = LoggerFactory.getLogger(GlobalCryptoSetup.class);

    static boolean loadedBouncyCastle = false;
    static Provider bouncyCastleProvider;

    public static boolean hasSecp256k1Curve() {
        // For now we just check if the java version is at least 8
        String[] version = System.getProperty("java.version").split("\\.");
        int major = Integer.parseInt(version[0]);
        int minor = Integer.parseInt(version[1]);
        return ((major > 1) || ((major > 0) && (minor > 7)));
    }

    public static  Provider getBouncyCastleProvider() { doSetup(); return bouncyCastleProvider; }

    public static synchronized void doSetup() {
        if (bouncyCastleProvider == null) {
            bouncyCastleProvider = new BouncyCastleProvider();
            // Make bouncycastle our default provider if we're running on a JVM version < 8
            // (earlier version don't support the EC curve we use for signatures)
            if (!hasSecp256k1Curve() && !loadedBouncyCastle) {
                loadedBouncyCastle = true;
                Security.insertProviderAt(bouncyCastleProvider, 1);
                logger.info("Using BouncyCastle instead of native provider to support secp256k1 named curve");
            }
        }
    }

    public GlobalCryptoSetup() { doSetup(); }
}
