package meerkat.voting;

import meerkat.protobuf.Voting.BallotSecrets;
import meerkat.protobuf.Voting.EncryptedBallot;
import meerkat.protobuf.Voting.PlaintextBallot;

public class VBMessage {

	public enum VBMessageType 
	{
		VB_TICK, 
		VB_CONTROLLER_RESPONSE,
		VB_ENCRYPTION_QUERY
	}

	private VBMessageType m_type;
	private EncryptedBallot m_encryptedBallot;
	private PlaintextBallot m_plaintextBallot;
	private BallotSecrets m_secrets;

	public static VBMessage newTick () {
		VBMessage retVal = new VBMessage();
		retVal.m_type = VBMessageType.VB_TICK;
		return retVal;
	}
	
	public VBMessageType getType () {
		return m_type;
	}
	
	public static VBMessage newEncryptionQuery (PlaintextBallot plaintextBallot) {
		VBMessage retVal = new VBMessage();
		retVal.m_type = VBMessageType.VB_ENCRYPTION_QUERY;
		retVal.m_plaintextBallot = plaintextBallot;
		return retVal;
	}

	public static VBMessage newControllerResponse (EncryptedBallot encryptedBallot, BallotSecrets ballotSecrets) {
		VBMessage retVal = new VBMessage();
		retVal.m_type = VBMessageType.VB_CONTROLLER_RESPONSE;
		retVal.m_encryptedBallot = encryptedBallot;
		retVal.m_secrets = ballotSecrets;
		return retVal;
	}
	
	public EncryptedBallot getEncryptedBallot () {
		assert getType() == VBMessageType.VB_CONTROLLER_RESPONSE;
		return m_encryptedBallot;
	}

	public BallotSecrets getSecrets () {
		assert getType() == VBMessageType.VB_CONTROLLER_RESPONSE;
		return m_secrets;
	}

	public PlaintextBallot getPlaintextBallot () {
		assert getType() == VBMessageType.VB_ENCRYPTION_QUERY;
		return m_plaintextBallot;
	}

	public boolean isEmptyMessage () {
		return (getType() == VBMessageType.VB_TICK);
	}
}
