package meerkat.voting;

import meerkat.protobuf.Voting.*;
import meerkat.crypto.Encryption;
import meerkat.protobuf.Crypto.*;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.List;
import java.util.Random;
import java.util.concurrent.ArrayBlockingQueue;

import com.google.protobuf.ByteString;



public class VotingBoothToy implements VotingBooth, Runnable {

	//private ElectionParams m_electionParams;		
	private EncryptionPublicKey m_ballotEncryptionKey;
	private List<BallotQuestion> l_questions;
	private BallotQuestion a_questions[];
	private BallotAnswer a_answers[];
	private BallotAnswerTranslationTable m_answerTranslationTable;
	
	private ArrayBlockingQueue<VBMessage> a_queue;
	static private int m_queueSize = 5;

	private VotingBooth.UI m_ui;
	
	//TODO: where do I use these?
	SignatureVerificationKey a_signatureKeys[];
	
	Encryption m_encryption;
	Random m_random;
	
	
	
	public VotingBoothToy () {
		a_queue = new ArrayBlockingQueue<VBMessage> (m_queueSize, true);
		
		m_encryption = new EncryptionFake(m_ballotEncryptionKey);
		m_random = new Random();
	}
	
	public void registerUI (UI ui) {
		m_ui = ui;
	}
	
	public void run () {
		System.err.println("VB controller is up and running");
		
		while (true) {
			VBMessage msg;
			try {
				msg = a_queue.take();
			}
			catch (InterruptedException e) {
				System.err.println("VB controller interrupted!!");
				break;
			}
			if (msg.getType() == VBMessage.VBMessageType.VB_ENCRYPTION_QUERY) {
				handleEncryptionQuery (msg);
			}
			else {
				System.err.println("VB controller unknown type of message!!");
				break;
			}
		}
	}
	
	
	private void handleEncryptionQuery (VBMessage msg) {
		assert msg.getType() == VBMessage.VBMessageType.VB_ENCRYPTION_QUERY;
		PlaintextBallot ptb = msg.getPlaintextBallot();
		
		// encrypt
		RerandomizableEncryptedMessage encMsg;
		EncryptionRandomness rnd = m_encryption.generateRandomness(m_random);
		try {
			// simulate taking a lot of time
			Thread.sleep(30000);
			encMsg = m_encryption.encrypt(ptb, rnd);
		}
		catch (InterruptedException e) {
			System.err.println("debug: VotingBoothToy encrypt interrupted!");
			return;
		}
		catch (IOException e) {
			System.err.println("debug: VotingBoothToy encrypt IOException!");
			return;
		}
		
		EncryptedBallot encBallot = EncryptedBallot.newBuilder()
				                         .setSerialNumber(ptb.getSerialNumber())
				                         .setData(encMsg)
				                         .build();
		RandomnessGenerationProof proof = RandomnessGenerationProof.newBuilder()
				                         .setData(ByteString.copyFrom(new byte[0]))
				                         .build();
		BallotSecrets secrets = BallotSecrets.newBuilder()
				                         .setPlaintextBallot(ptb)
				                         .setEncryptionRandomness(rnd)
				                         .setProof(proof)
				                         .build();
		
		m_ui.commitToEncryptedBallot(encBallot, secrets);
	}
	
	
	/* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth#init(meerkat.protobuf.Voting.ElectionParams, meerkat.protobuf.Voting.BoothParams)
	 */
	@Override
	public void init(ElectionParams globalParams, BoothParams boothParams) {
		System.err.println ("debug VB: init.");
		this.m_ballotEncryptionKey = globalParams.getBallotEncryptionKey();
		this.l_questions =  globalParams.getQuestionsList();
		
		this.m_answerTranslationTable = globalParams.getAnswerTranslationTable();

		List<SignatureVerificationKey> l_signatureKeys =  boothParams.getPscVerificationKeysList();
		a_signatureKeys = new SignatureVerificationKey[l_signatureKeys.size()];
		int i = 0;
		for (SignatureVerificationKey q: l_signatureKeys) {
			a_signatureKeys[i] = q;
			++i;
		}

	}

	public void registerVBUI (VotingBooth.UI ui)
	{
		m_ui = ui;
	}
	/* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth#submitBallot(meerkat.protobuf.Voting.PlaintextBallot)
	 */
	@Override
	public void submitBallot(PlaintextBallot ballot) {
		VBMessage msg = VBMessage.newEncryptionQuery(ballot);
		try {
			a_queue.put(msg);
		}
		catch (InterruptedException e) {
			System.err.println("SubmitBallot interrupted!!");
		}
	}

	/* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth#cancelBallot()
	 */
	/*@Override
	public void cancelBallot() {
		// TODO Auto-generated method stub

	}*/

	/* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth#voterCastOrAudit(boolean)
	 */
	/*@Override
	public void voterCastOrAudit(boolean castVote) {
		// TODO Auto-generated method stub

	}*/

}



/*

//A ballot question. This is an opaque
//data type that is parsed by the UI to display
//the question.
message BallotQuestion {
 bytes data = 1;
}

//An answer to a specific ballot question.
//The answer is a vector of signed integers,
//to encompass voting schemes such as ranked voting
//and STV.
message BallotAnswer {
 repeated sint64 answer = 1 [packed=true];
}

message EncryptedBallot {
    uint64 serialNumber = 1; // Ballot serial number

    RerandomizableEncryptedMessage data = 2;
}

message BallotSecrets {
    PlaintextBallot plaintext_ballot = 1;

    EncryptionRandomness encryption_randomness = 2;
    RandomnessGenerationProof proof = 3;
}

message BoothParams {
 repeated SignatureVerificationKey pscVerificationKeys = 1;

}

//A table to translate to and from compactly encoded answers
//and their human-understandable counterparts.
//This should be parsable by the UI
message BallotAnswerTranslationTable {
 bytes data = 1;
}

message ElectionParams {
 // TODO: different sets of keys for different roles?
 //repeated SignatureVerificationKey trusteeVerificationKeys = 1;

 // How many trustees must participate in a signature for it to be considered valid.
 //uint32 trusteeSignatureThreshold = 2;

 // The key used to encrypt ballots. The corresponding private key
 // is shared between the trustees.
 EncryptionPublicKey ballotEncryptionKey = 3;

 // Verification keys for valid mixers.
 //repeated SignatureVerificationKey mixerVerificationKeys = 4;

 // How many mixers must participate for the mixing to be considered valid
 //uint32 mixerThreshold = 5;

 // Candidate list (or other question format)
 repeated BallotQuestion questions = 6;

 // Translation table between answers and plaintext encoding
 BallotAnswerTranslationTable answerTranslationTable = 7;

}
*/