/**
 * 
 */
package meerkat.voting;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Date;
import java.util.List;
import java.util.StringTokenizer;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ArrayBlockingQueue;

import com.google.protobuf.ByteString;

import meerkat.protobuf.Voting.*;
import meerkat.voting.VotingBooth.UI;

/**
 * @author hai
 *
 */
public class VotingBoothToyConsoleUI implements UI, Runnable {

	private BufferedReader m_in;
	private VotingBooth m_vbController;
	//private SharedPlaintextBallotMessage m_sharedPlaintext;
	//private SharedEncryptedBallotMessage m_sharedEncrypted;
	private ArrayBlockingQueue<VBMessage> a_queue;
	static private int m_queueSize = 5;
	private BallotQuestion a_questions[];
	private BallotQuestionNew a_questionsNew[];
	private int m_serialNumber;
	private PlaintextBallot m_plaintextBallot;
	private EncryptedBallot m_encryptedBallot;
	private BallotSecrets m_ballotSecrets;
	private int m_waitForControllerMillisecTimeout = 10;
	private BallotAnswerTranslationTable m_answerTranslationTable;

	
	public VotingBoothToyConsoleUI () {
		a_queue = new ArrayBlockingQueue<VBMessage> (m_queueSize, true);
	}
	
	public VotingBoothToyConsoleUI(ElectionParams globalParams) {
		m_serialNumber = 0;
		
		this.m_answerTranslationTable = globalParams.getAnswerTranslationTable();

		List<BallotQuestion> l_questions =  globalParams.getQuestionsList();
		a_questions = new BallotQuestion[l_questions.size()];
		m_in = new BufferedReader(new InputStreamReader(System.in));
		
		int i = 0;
		for (BallotQuestion q: l_questions) {
			a_questions[i] = q;
			++i;
		}
	}
	


	/* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth.UI#votingBegin()
	 */
	@Override
    public void votingBegin() {
		System.err.println ("UI debug: preparing console UI for a new user.");
		
		// clear history from memory
		eraseEncryption();
		erasePlaintext();
		
		System.out.println ("UI screen: Welcome. Press to start");
		readInputLine();
		
		++ m_serialNumber;
		
		runVotingFlow ();
	}
	
	private void runVotingFlow () {
		
		try {
		
			boolean votingOccured = listQuestionsToUserAndGetAnswers ();
			
			if (! votingOccured)
			{
				// cancel vote;
				return;
			}
			else {
				printBallotFlow();
			}
			
		}
		catch (IOException e) {
			System.err.println("UI debug: IO error");
		}
	}

	
	private void sendBallotToPrinterAndWaitToEnd () {
		System.err.println("UI debug: sendBallotToPrinterAndWaitToEnd: still not multithreaded...");
		System.out.println("UI screen: printing ballot wait for printing to end");
		System.out.println("UI printer: printing ballot");
	}
	
	// returns true if cast or cancelled
	private boolean castOrAudit () throws IOException{
		System.out.println("UI screen: (c)ast or (a)udit?");
		String choice = readInputLine();
		if (choice.equals("a") || choice.equals("audit")) {
			System.err.println ("UI debug: voter decided to audit");
			System.out.println("UI screen: printing audit wait for printing to end");
			System.out.println("UI printer: printing audit");
			System.out.println("Do you still want to keep this vote? Type 'y' for casting/auditing the same vote. Type 'n' tp start from scratch");
			String choice2 = readInputLine();
			if (choice2.equals("y") || choice.equals("yes")) {
				return false;
			}
			else if (choice2.equals("n") || choice.equals("no")) {
				return true;
			}
			else {
				throw new IOException ();
			}
		}
		else if (choice.equals("c") || choice.equals("cast") || choice.equals("cancel")) {
			System.err.println ("UI debug: voter decided to cast vote.");
			System.out.println("UI screen: wait for printer to finish");				
			System.out.println("UI printer: printing empty space till end of page");
			return true;
		}
		else {
			throw new IOException ();
		}
	}
	
	
	private void eraseEncryption () {
		//TODO: should we clean memory stronger?
		if (m_encryptedBallot != null) {
			m_encryptedBallot = null;
		}
		if (m_ballotSecrets != null) {
			m_ballotSecrets = null;
		}
	}

	private void erasePlaintext () {
		//TODO: should we clean memory stronger?
		if (m_plaintextBallot != null) {
			m_plaintextBallot = null;
		}
	}

	private void printBallotFlow () throws IOException{
		System.err.println("UI debug: printBallotFlow currently not multithreaded... will fix it...");
		
		boolean ended = false;
		while (!ended) {
			sendBallotToControllerForEncryptionAndWaitForResponse ();
			sendBallotToPrinterAndWaitToEnd();
			ended = castOrAudit();
			eraseEncryption();
		}
		erasePlaintext();
	}
	
	/* (non-Javadoc)
	 * @see meerkat.voting.VotingBooth.UI#commitToEncryptedBallot(meerkat.voting.EncryptedBallot)
	 */
	@Override
    public void commitToEncryptedBallot(EncryptedBallot encryptedBallot, BallotSecrets secrets) {
		try {
			a_queue.put (VBMessage.newControllerResponse(encryptedBallot, secrets));
		}
		catch (InterruptedException e) {
			System.err.println ("Interrupted in VotingBoothToyConsoleUI.commitToEncryptedBallot");
		}
	}



	public void run () {		
		System.out.println("UI screen: Initializing the magic");
		
		System.out.println("UI screen: press something to start the system");
		readInputLine();

		while (true) {
			votingBegin();
		}
	}
	
	

	public void registerVBController (VotingBooth vb)
	{
		m_vbController = vb;
		//m_sharedPlaintext = sharedPlaintext;
		//m_sharedEncrypted = sharedEncrypted;
	}
	
	/*
	 * returns true if voting finished successfully
	 * false if cancelled in the middle
	 */
	private boolean listQuestionsToUserAndGetAnswers()
	{
		PlaintextBallot.Builder ptbb = PlaintextBallot.newBuilder();

		ptbb.setSerialNumber(m_serialNumber);
		
		int index = 0;
		while (index < a_questions.length) {
			BallotQuestion q = a_questions[index];
			printQuestion(index, q);
			System.out.println("UI screen: Enter your answer. You can also type 'back' or 'cancel'");
			String s = readInputLine();
			
			if (s.equals("cancel") || (index == 0 && s.equals("back"))) {
				erasePlaintext();
				return false;
			}

			if (s.equals("back")) {
				--index;
				continue;
			}
			
			BallotAnswer answer = translateStringAnswerToProtoBufMessageAnswer (s);
			ptbb.setAnswers(index, answer);
		}
		
		m_plaintextBallot = ptbb.build();
		return true;
	}

	
	private String readInputLine () {
		String s;
		try {
			s = this.m_in.readLine();
		}
		catch (IOException e) {
			System.err.println("UI debug: VotingBegin(): some error with reading input. " + e);
			return null;
		}

		return s;
	}
	
	private String getHexData (ByteString data) {
		String s = "";
		for (Byte b : data) {
			s += "0123456789ABCDEF".charAt((int)b / 16);
			s += "0123456789ABCDEF".charAt((int)b % 16);			
		}
		return s;
	}
	
	private void printHexData (ByteString data) {
		String s = getHexData(data);
		System.out.println(s);
	}
	
	private void printQuestion (int i, BallotQuestion q) {
		System.out.println("UI screen: question number " + i);
		System.out.println(q.getData());
		printHexData (q.getData());	
	}
	
	private BallotAnswer translateStringAnswerToProtoBufMessageAnswer (String s) {
		BallotAnswer.Builder bab = BallotAnswer.newBuilder();
	    StringTokenizer st = new StringTokenizer(s);
	    int index = 0;
	    while (st.hasMoreTokens()) {
	    	++index;
			bab.setAnswer(index, Integer.parseInt(st.nextToken()));
	    }

		BallotAnswer ba = bab.build();
		return ba;
	}
	
	public void tick () {
		// adds a 'tick'. If queue is full, do nothing
		a_queue.add (VBMessage.newTick());
	}
	
	private void sendBallotToControllerForEncryptionAndWaitForResponse () {
		
		class TickerTask extends TimerTask {
			private VotingBoothToyConsoleUI m_ui;
			public TickerTask (VotingBoothToyConsoleUI ui) {
				m_ui = ui;
			}
			  
			@Override
			public void run() {
			    m_ui.tick();
			} 			
		}
		
		eraseEncryption();
		System.out.println("UI screen: Please wait for encryption");
		m_vbController.submitBallot(m_plaintextBallot);
		Timer timer = new Timer();
		timer.scheduleAtFixedRate(new TickerTask(this), new Date(), m_waitForControllerMillisecTimeout);
		
		while (m_encryptedBallot == null)
		{
			VBMessage msg = null;
			try {
				msg = a_queue.take();
			}
			catch (InterruptedException e) {
				System.err.println("VotingBoothToyConsoleUI.sendBallotToControllerForEncryptionAndWaitForResponse  interrupted!");
				return;
			}
			if (msg.isEmptyMessage()) {
				System.out.print(".");
			}
			else {
				m_encryptedBallot = msg.getEncryptedBallot();
				if (m_encryptedBallot.getSerialNumber() != m_plaintextBallot.getSerialNumber())
				{
					eraseEncryption();
				}
			}
		}
		
		System.out.println("\nUI debug: Received EncryptedBallot");
		timer.cancel();
		timer.purge();
		
	}
}
